/*
 * CFINGERD 1.1.0
 * Main Routine
 */

#include "cfingerd.h"
#include "proto.h"

CONFIG prog_config;
char *remote_addr, *localhost;
BOOL local_finger, emulated;

/*
 * HAS_MATCH
 *
 * Check to see if there's an occurance of "match" within "string".  You're
 * probably saying that I could have used strncmp, or something else
 * similar.  Not true - because I'd need about 3 calls in here, and this
 * routine does everything I need in *1* routine.  :)
 */
BOOL has_match(char *string, char *match)
{
    int i, x;

    for (i = 0; i < strlen(string); i++)
	for (x = 0; x < strlen(match); x++)
	    if (string[i] == match[x])
		return TRUE;

    return FALSE;
}

/*
 * MAIN
 *
 * The main dispatcher for cfingerd.
 */
void main(int argc, char *argv[])
{
    char username[80], syslog_str[80];
    int un_type;

    /* Initialize CFINGERD */
    start_handler();
    start_cfingerd();

    /* Checn for any command-line options */
    if (argc > 1) {
	(void) check_options(argc, argv);
	strcpy(username, argv[2]);
    }

    /* Read the configuration, and check status of the program so far */
    read_configuration();
    check_stats();

    /* Change program priority to highest priority when running */
    if (nice(-20) == -1) {
	syslog(LOG_ERR, "%s", prog_config.p_strings[D_NICE_FATAL]);
	exit(0);
    }

    /* Make sure there is actually data waiting in the finger port */
    if (!emulated)
	if (!fgets(username, sizeof(username), stdin)) {
	    syslog(LOG_ERR, "%s: %s", remote_addr, strerror(errno));
	    closelog();
	    exit(PROGRAM_SYSLOG);
	}

    /* If there's a username, trim the CRLF pair */
    if (!emulated)
	sscanf(username, "%[^\r\n]\r\n", username);

    /* If we're not doing emulated stuff, we can assume that we are running
       either as a daemon, or under INETD.  In that case... */
    if (!emulated) {
	struct sockaddr_in socket_addr;
	struct hostent *host_ent;
	int psize = 0;

	/* Can't run from command line (but this should already be checked) */
	psize = sizeof(socket_addr);
	if (getpeername(0, (struct sockaddr *) &socket_addr, &psize)) {
	    printf("Internal error - not running as either a daemon or under INETD.\n");
	    printf("Fatal - getpeername: %s\n", strerror(errno));
	    closelog();
	    exit(PROGRAM_BUG);
	}

	/* Get our host entry */
	host_ent = (struct hostent *) gethostbyaddr((char *) &socket_addr.sin_addr,
		   sizeof(socket_addr.sin_addr), AF_INET);

	/* And get our local-host name */
#ifndef	ACTUAL_HOSTNAME
	localhost = get_localhost();
#else
	localhost = ACTUAL_HOSTNAME;
#endif

	/* Make sure we can get the remote host's address name */
	if (host_ent == NULL) {
	    remote_addr = inettos(socket_addr.sin_addr.s_addr);
	    syslog(LOG_WARNING, "%s %s", 
		prog_config.p_strings[D_IP_NO_MATCH],
		remote_addr);
	    if (!(prog_config.config_bits2 & SHOW_IP_MATCH))
		CF_ERROR(E_NOIP);
	} else
	    remote_addr = (char *) host_ent->h_name;

	/* And find out if this is a local finger */
	if (!strncasecmp(remote_addr, "127.0.0.1", 9) ||
	    !strncasecmp(remote_addr, "localhost", 9) ||
	    !strncasecmp(remote_addr, localhost, strlen(localhost)))
	    local_finger = TRUE;
	else
	    local_finger = FALSE;
    } else
	local_finger = TRUE;

    /* We're emulated, so assume we're fingering from loopback */
    if (emulated) {
	local_finger = TRUE;
#ifndef	ACTUAL_LOOPBACK
	remote_addr = "127.0.0.1";
#else
	remote_addr = ACTUAL_LOOPBACK;
#endif
    }

    /* Now, let's check to make sure this site is trusted */
    if ((!local_finger) && !emulated)
	local_finger = check_trusted(remote_addr);

    if ((!local_finger) && !emulated)
	check_rejected(remote_addr);

    /* Check the finger information coming in and return its type */
    un_type = process_username(username);

    /* And check the process the information here. */
    switch(un_type) {
	case U_INTERNAL:
	    if (!emulated) {
		sprintf(syslog_str, "%s fingered (internal) from %s", username,
		    remote_addr);
		syslog(LOG_NOTICE, (char *) syslog_str);
	    }

	    handle_internal(username);
	    break;
	case U_STANDARD:
	    if (!emulated) {
		if (strncmp(username, "root", 4))
		    sprintf(syslog_str, "%s fingered from %s", username, remote_addr);
		else
		    sprintf(syslog_str, "%s fingered from %s",
			prog_config.p_strings[D_ROOT_FINGER], remote_addr);

		syslog(LOG_NOTICE, (char *) syslog_str);
	    }

	    handle_standard(username);
	    break;
	case U_FAKEUSER:
	    if (!emulated) {
		sprintf(syslog_str, "%s %s from %s", username, 
		    prog_config.p_strings[D_FAKE_USER], remote_addr);

		syslog(LOG_NOTICE, (char *) syslog_str);
	    }

	    handle_fakeuser(username);
	    break;
	case U_USERLIST:
	    if (!emulated)
		syslog(LOG_NOTICE, "%s from %s", prog_config.p_strings[D_ULIST_FINGER],
		    remote_addr);

	    handle_userlist();
	    break;
    }
}
